"use strict";
var rokot_mq_rabbit_1 = require("rokot-mq-rabbit");
var rokot_log_1 = require("rokot-log");
var Promise = require("bluebird");
var rokot_test_1 = require("rokot-test");
var logger = rokot_log_1.ConsoleLogger.create("rabbitmq-tests", { level: "trace" });
var config = { host: "127.0.0.1", port: 5672 };
var invalidConfig = { host: "127.0.0.1", port: 5670 };
var queueName = "TestPC";
describe("RabbitMQSingletonConnectionFactory Test", function () {
    it("Singleton Connection Factory should return singleton connection", function () {
        var f = new rokot_mq_rabbit_1.RabbitMqSingletonConnectionFactory(logger, config);
        return Promise.all([f.create(), f.create(), f.create()]).then(function (cons) {
            rokot_test_1.expect(cons).to.exist;
            rokot_test_1.expect(cons.length).to.eq(3);
            cons.forEach(function (con, i) {
                rokot_test_1.expect(con).to.exist;
                if (i > 0) {
                    rokot_test_1.expect(cons[0]).to.equal(con);
                }
            });
        });
    });
});
describe("RabbitMq Test", function () {
    it("ConnectionFactory: Invalid Connection config should fail create", function () {
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, invalidConfig);
        return rokot_test_1.expect(factory.create()).to.eventually.be.rejected.then(function (v) {
            rokot_test_1.expect(v).to.exist;
            rokot_test_1.expect(v.code).to.eq('ECONNREFUSED');
        });
    });
    it("RabbitMqConsumer: Invalid Connection config should fail subscribe", function () {
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, invalidConfig);
        var consumer = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        return rokot_test_1.expect(consumer.subscribe(queueName, function (m) { })).to.eventually.be.rejected.then(function (v) {
            rokot_test_1.expect(v).to.exist;
            rokot_test_1.expect(v.code).to.eq("ECONNREFUSED");
        });
    });
    it("RabbitMqProdcuer: Invalid Connection config should fail publish", function () {
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, invalidConfig);
        var producer = new rokot_mq_rabbit_1.RabbitMqProducer(logger, factory);
        return rokot_test_1.expect(producer.publish(queueName, function (m) { })).to.eventually.be.rejected.then(function (v) {
            rokot_test_1.expect(v).to.exist;
            rokot_test_1.expect(v.code).to.eq("ECONNREFUSED");
        });
    });
    it("Consumer should subscribe and dispose ok with simple queue name", function () {
        var spy = rokot_test_1.sinon.spy();
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, config);
        var consumer = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        return consumer.subscribe(queueName, spy).then(function (s) { return Promise.delay(500, s); })
            .then(function (disposer) {
            rokot_test_1.expect(disposer, "disposer should exist").to.exist;
            rokot_test_1.expect(spy.callCount).to.be.eq(0, "Consumer spy should not have been called");
            return rokot_test_1.expect(disposer()).to.eventually.be.fulfilled;
        });
    });
    it('Consumer should recieve message from Producer', function () {
        var spy = rokot_test_1.sinon.spy();
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, config);
        var consumer = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        return consumer.subscribe(queueName, spy).then(function (disposer) {
            var producer = new rokot_mq_rabbit_1.RabbitMqProducer(logger, factory);
            var msg = { data: "time", value: new Date().getTime() };
            return rokot_test_1.expect(producer.publish(queueName, msg)).to.eventually.be.fulfilled
                .then(function () { return Promise.delay(500); })
                .then(function () {
                rokot_test_1.expect(spy.callCount).to.be.eq(1, "Consumer spy should have been called once");
                rokot_test_1.expect(spy.firstCall.args).to.exist;
                rokot_test_1.expect(spy.firstCall.args.length).to.be.eq(1, "Spy should have been called with message argument");
                var consumedMsg = spy.firstCall.args[0];
                rokot_test_1.expect(consumedMsg.data).to.exist;
                rokot_test_1.expect(consumedMsg.data).to.be.eq(msg.data, "data property mismatch");
                rokot_test_1.expect(consumedMsg.value).to.exist;
                rokot_test_1.expect(consumedMsg.value).to.be.eq(msg.value, "value property mismatch");
                disposer();
            });
        });
    });
    it('Consumer should recieve string message from Producer', function () {
        var spy = rokot_test_1.sinon.spy();
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, config);
        var consumer = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        return consumer.subscribe(queueName, spy).then(function (disposer) {
            var producer = new rokot_mq_rabbit_1.RabbitMqProducer(logger, factory);
            var msg = "good";
            return rokot_test_1.expect(producer.publish(queueName, msg)).to.eventually.be.fulfilled
                .then(function () { return Promise.delay(500); })
                .then(function () {
                rokot_test_1.expect(spy.callCount).to.be.eq(1, "Consumer spy should have been called once");
                rokot_test_1.expect(spy.firstCall.args).to.exist;
                rokot_test_1.expect(spy.firstCall.args.length).to.be.eq(1, "Spy should have been called with message argument");
                var consumedMsg = spy.firstCall.args[0];
                rokot_test_1.expect(consumedMsg).to.exist;
                rokot_test_1.expect(consumedMsg).to.be.eq(msg, "data property mismatch");
                disposer();
            });
        });
    });
    it('Create Consumer and Producer seperately', function () {
        var spy = rokot_test_1.sinon.spy();
        var factory = new rokot_mq_rabbit_1.RabbitMqConnectionFactory(logger, config);
        var consumer = new rokot_mq_rabbit_1.RabbitMqConsumer(logger, factory);
        var producer = new rokot_mq_rabbit_1.RabbitMqProducer(logger, factory);
        var msg = { data: "time", value: new Date().getTime() };
        rokot_test_1.expect(producer.publish(queueName, msg)).to.eventually.be.fulfilled.then(function (v) {
            console.log(v);
        });
        consumer.subscribe(queueName, spy)
            .then(function (disposer) {
            rokot_test_1.expect(disposer).to.exist;
            rokot_test_1.expect(spy.callCount).to.be.eq(1, "Consumer spy should have been called once");
            rokot_test_1.expect(spy.firstCall.args).to.exist;
            rokot_test_1.expect(spy.firstCall.args.length).to.be.eq(1, "Spy should have been called with message argument");
            var consumedMsg = spy.firstCall.args[0];
            rokot_test_1.expect(consumedMsg.data).to.exist;
            rokot_test_1.expect(consumedMsg.data).to.be.eq(msg.data, "data property mismatch");
            rokot_test_1.expect(consumedMsg.value).to.exist;
            rokot_test_1.expect(consumedMsg.value).to.be.eq(msg.value, "value property mismatch");
            console.log("Consumed message" + consumedMsg);
            return rokot_test_1.expect(disposer()).to.eventually.be.fulfilled;
        });
    });
});
//# sourceMappingURL=rabbitmq-tests.js.map